;=======================================================
; Reflecting Block
; Description: This sprite acts as a block that reflects
; enemy projectiles such as Pitchin' Chuck's baseballs.
; Projectiles reflected are only extended sprites.
;
; By Erik557
;=======================================================

       !FireballInt  = 1                  ; set to 0 if you don't want mario's fireballs
                                          ; to be reflected
       !Tile         = $C0

       !FreeRAM             = $0F5E       ; 10 bytes
       !RAM_SpriteYLo       = $D8
       !RAM_SpriteXLo       = $E4
       !RAM_SpriteStatus    = $14C8
       !RAM_SpriteYHi       = $14D4
       !RAM_SpriteXHi       = $14E0

;=================================
; INIT and MAIN Wrappers
;=================================

print "MAIN ",pc
       PHB
       PHK
       PLB
       JSR ReflectingBlock
       PLB
print "INIT ",pc
       RTL


;========================
; Main routine
;========================

ReflectingBlock:
       JSR Graphics         ; draw the sprite
       JSL $01B44F          ; make solid

       ; basically a clone of 01a7dc but with extended sprites
       LDA $14              ;\
       AND #$07             ; | return every other frame
       ORA $9D              ; | or if sprites are locked
       BNE .Return          ;/

       ; check for contact with an extended sprite
       if !FireballInt == 1
              LDY #$09      ; oh yeah check fireballs too :^)
       else
              LDY #$07      ; 8 slots to check
       endif
.loop
       LDA $170B,y          ;\ if number not zero, interact
       BNE .Interact        ;/
.CheckNext
       DEY                  ;\ return if looped through all
       BPL .loop            ;/
.Return
       RTS

.Interact
       CMP #$02             ;\
       BCC .CheckNext       ; | only check sprites 02 through 0D
       CMP #$11             ; | (and 11 too)
       BEQ .Process         ; |
       CMP #$0E             ; |
       BCS .CheckNext       ;/

.Process
       LDA $171F,y          ;\
       SEC                  ; |
       SBC !RAM_SpriteXLo,x ; | Too far horizontally, return
       CLC                  ; |
       ADC #$50             ; |
       CMP #$A0             ; |
       BCS .CheckNext       ;/
       LDA $1715,y          ;\
       SEC                  ; |
       SBC !RAM_SpriteYLo,x ; | Too far vertically, return
       CLC                  ; |
       ADC #$60             ; |
       CMP #$C0             ; |
       BCS .CheckNext       ;/

       LDA $1632,x          ;\
       EOR $1779,y          ; | Not on the same scenery, return
       BNE .CheckNext       ;/

       LDX $170B,y          ; x = extended sprite number
       LDA $171F,y          ;\
       CLC                  ; | $00 - Clipping X-displacement low
       ADC.l $02A4E7,x      ; |
       STA $00              ;/
       LDA $1733,y          ;\
       ADC #$00             ; | $08 - Clipping X-displacement high
       STA $08              ;/
       LDA.l $02A4FF,x      ;\  $02 - Clipping width
       STA $02              ;/
       LDA $1715,y          ;\
       CLC                  ; | $01 - Clipping Y-displacement low
       ADC.l $02A4F3,x      ; |
       STA $01              ;/
       LDA $1729,y          ;\
       ADC #$00             ; | $09 - Clipping Y-displacement high
       STA $09              ;/
       LDA.l $02A50B,x      ;\  $03 - Clipping height
       STA $03              ;/

       LDX $15E9            ; return sprite index
       JSL $03B69F          ; get sprite's clipping
       JSL $03B72B          ; check for contact
       BCC .CheckNext       ; return if no contact

       LDA $7FAB28,x
       ASL
       TAX
       JSR (.ReflectDir,x)
       LDX $15E9
       JMP .CheckNext

.ReflectDir
       dw LeftUp
       dw LeftDown
       dw RightUp
       dw RightDown
       dw Reflect

LeftUp:
RightUp:
       LDA #$01             ;\ swapped speed flag
       STA !FreeRAM,y       ;/
       LDA $1747,y
       XBA
       LDA $173D,y
       EOR #$FF
       INC
       STA $1747,y
       XBA
       STA $173D,y
       RTS

LeftDown:
       LDA #$01             ;\ swapped speed flag
       STA !FreeRAM,y       ;/
       LDA $1747,y
       XBA
       LDA $173D,y
       STA $1747,y
       XBA
       STA $173D,y
       RTS

RightDown:
       LDA #$01             ;\ swapped speed flag
       STA !FreeRAM,y       ;/
       LDA $1747,y
       XBA
       LDA $173D,y
       EOR #$FF
       INC
       STA $1747,y
       XBA
       EOR #$FF
       INC
       STA $173D,y
       RTS

Reflect:
       LDA $1747,y
       EOR #$FF
       INC
       STA $1747,y
       RTS

;==================
; Graphics routine
;==================

Graphics:
       JSR GetDrawInfo
       REP #$20             ; 1 byte less and same cycle amount so may as well
       LDA $00              ;\ xy pos relative to screen border
       STA $0300,y          ;/
       SEP #$20             ;
       LDA #!Tile           ;\ tilemap
       STA $0302,y          ;/
       LDA $15F6,x          ;\
       ORA $64              ; | properties
       STA $0303,y          ;/
       LDY #$02             ; 16x16
       LDA #$00             ; 1 tile
       JSL $01B7B3          ; finish OAM write
       RTS

;================================================
  GetDrawInfo:            ;| - Optimized version
;================================================

       LDA !RAM_SpriteXHi,x
       XBA
       LDA !RAM_SpriteXLo,x
       REP #$20
       SEC
       SBC $1A
       STA $00
       CLC
       ADC.w #$0040
       CMP.w #$0180
       SEP #$20
       LDA $01
       BEQ +
       LDA #$01
+      STA $15A0,x
       TDC
       ROL A
       STA $15C4,x
       BNE .Invalid
       LDA !RAM_SpriteYHi,x
       XBA
       LDA $1662,x
       AND #$20
       BEQ .CheckOnce
.CheckTwice
       LDA !RAM_SpriteYLo,x
       REP #$21
       ADC.w #$001C
       SEC
       SBC $1C
       SEP #$20
       LDA !RAM_SpriteYHi,x
       XBA
       BEQ .CheckOnce
       LDA #$02
.CheckOnce
       STA $186C,x
       LDA !RAM_SpriteYLo,x
       REP #$21
       ADC.w #$000C
       SEC
       SBC $1C
       SEP #$21
       SBC #$0C
       STA $01
       XBA
       BEQ .OnScreenY
       INC $186C,x
.OnScreenY
       LDY $15EA,x
       RTS

.Invalid
       PLA
       PLA
       RTS


